 part of '../firebase_functions_rest.dart';

class Functionbase {

  String appId;
  String region;
  String idtoken;
  String function;
  bool https;

  /// The Functionbase is specified by the [appId] (For example "someapp-db2n")
  /// a [function] (For example users) and the [region] of the function(Defaults to "us-central1")
  /// Eventually you want to add a authentication header when using Firebase Auth
  /// to achieve this, you should provide a [token]
  /// If you do not want to use [https] set the parameter to false
  Functionbase(this.appId, this.function, {this.region = "us-central1", this.idtoken, this.https = true});

  /// Invokes a post request for [method] with the content [body]
  /// If you dont want to use add a auth header, set [auth] to false
  /// If you want to specify a query, set [query]
  /// If you want to modify the request before sending, specify [prepareRequest]
  ///
  /// Returns the response of the function as a UTF8 String
  Future<String> post(String method, String body, {bool auth = true, Map<String,String> query, String contentType, Function(HttpClientRequest) prepareRequest}) async {
    _checkAuth(auth);
    Uri uri = _parseUri(method, query);
    HttpClient httpClient = HttpClient();
    if (auth) {
      HttpClientResponse response = await httpClient.postUrl(uri).then((HttpClientRequest request) {
        request.headers.add("Authorization", "Bearer $idtoken");
        request.headers.add("Content-Type", contentType);
        request.write(body);

        if (prepareRequest != null) {
          prepareRequest(request);
        }

        return request.close();
      });
      return await Utf8Codec().decodeStream(response);
    } else {
      HttpClientResponse response = await httpClient.postUrl(uri).then((HttpClientRequest request) {
        request.headers.add("Content-Type", contentType);
        request.write(body);

        if (prepareRequest != null) {
          prepareRequest(request);
        }

        return request.close();
      });
      return await await Utf8Codec().decodeStream(response);
    }
  }

  /// Invokes a get request for [method]
  /// If you dont want to use add a auth header, set [auth] to false
  /// If you want to specify a query, set [query]
  /// If you want to modify the request before sending, specify [prepareRequest]
  ///
  /// Returns the function response as a UTF8 String
  Future<String> get(String method, {bool auth = true, Map<String,String> query, String contentType, Function(HttpClientRequest) prepareRequest}) async {
    _checkAuth(auth);
    Uri uri = _parseUri(method, query);
    HttpClient httpClient = HttpClient();
    if (auth) {
      HttpClientResponse response = await httpClient.getUrl(uri).then((HttpClientRequest request) {
        request.headers.add("Authorization", "Bearer $idtoken");
        request.headers.add("Content-Type", contentType);

        if (prepareRequest != null) {
          prepareRequest(request);
        }

        return request.close();
      });
      return await Utf8Codec().decodeStream(response);
    } else {
      HttpClientResponse response = await httpClient.getUrl(uri).then((HttpClientRequest request) {
        request.headers.add("Content-Type", contentType);

        if (prepareRequest != null) {
          prepareRequest(request);
        }

        return request.close();
      });
      return await Utf8Codec().decodeStream(response);
    }
  }

  /// Refer to post()
  Future<String> postJson(String method, Map<String,dynamic> json, {bool auth = true, Map<String,String> query}) async {
    return await post(method, jsonEncode(json), auth: auth, query: query, contentType: "application/json");
  }

  /// Refer to post()
  Future<Map<String,dynamic>> getJson(String method, {bool auth = true, Map<String,String> query}) async {
    return jsonDecode(await get(method, auth: auth,  query: query, contentType: "application/json"));
  }

  Uri _parseUri(String method, Map<String,String> query) {
    return Uri.parse(_getProtocol() + "$region-$appId.cloudfunctions.net/$function/$method" + _parseQueryString(query));
  }

  String _parseQueryString(Map<String,String> query) {
    if (query == null) {
      return "";
    } else {
      return "?" + query.entries.map((MapEntry<String,String> entry) {
        return entry.key + "=" + entry.value;
      }).join("&");
    }
  }

  void _checkAuth(bool auth) {
    if (idtoken == null && auth) {
      throw Exception("a rest function was invoked with auth = true, altrough no token was given in the constructor");
    }
  }
  
  String _getProtocol() {
    return https ? "https://" : "http://";
  }
}